/***************************************************************************//**
 *   @file   ADIS16227.c
 *   @brief  Implementation of ADIS16227 Driver.
 *   @author DBogdan (dragos.bogdan@analog.com)
********************************************************************************
 * Copyright 2012(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
********************************************************************************
 *   SVN Revision: 491
*******************************************************************************/

/******************************************************************************/
/* Include Files                                                              */
/******************************************************************************/
#include "ADIS16227.h"      // ADIS16227 definitions.
#include "Communication.h"  // Communication definitions.

/***************************************************************************//**
 * @brief Reads the value of a register.
 *
 * @param registerAddress - Address of the register.
 *
 * @return registerValue - Value of the register.
*******************************************************************************/
unsigned short ADIS16227_GetRegister16Value(unsigned char registerAddress)
{
    unsigned char   readData[3]     = {0, 0, 0};
    unsigned short  registerValue   = 0;
    
    readData[0] = 0x01;    // Chip Select.
    readData[1] = registerAddress;
    readData[2] = 0x00;
    SPI_Read(readData,2);
    readData[0] = 0x01;    // Chip Select.
    readData[1] = 0x00;
    readData[2] = 0x00;
    SPI_Read(readData,2);
    registerValue = ((unsigned short)readData[0] << 8) + readData[1];
    
    return(registerValue);
}

/***************************************************************************//**
 * @brief Writes 1 byte into a register.
 *
 * @param registerAddress - Address of the register.
 * @param registerValue - Data value to write.
 *
 * @return None.
*******************************************************************************/
void ADIS16227_SetRegister8Value(unsigned char registerAddress,
                                 unsigned char registerValue)
{
    unsigned char writeData[3]  = {0, 0, 0};
    
    writeData[0] = 0x01;    // Chip Select.
    writeData[1] = 0x80 + registerAddress;
    writeData[2] = registerValue;
    SPI_Write(writeData,2);
}

/***************************************************************************//**
 * @brief Writes 2 bytes into a register.
 *
 * @param registerAddress - Address of the register.
 * @param registerValue - Data value to write.
 *
 * @return None.
*******************************************************************************/
void ADIS16227_SetRegister16Value(unsigned char registerAddress,
                                  unsigned short registerValue)
{
    unsigned char writeData[2]  = {0, 0};
    
    writeData[0] = (unsigned char)(registerValue & 0x00FF);
    writeData[1] = (unsigned char)((registerValue & 0xFF00) >> 8);
    ADIS16227_SetRegister8Value(registerAddress, writeData[0]);
    ADIS16227_SetRegister8Value(registerAddress + 1, writeData[1]);
}

/***************************************************************************//**
 * @brief Initializes the SPI peripheral and checks if the ADIS16227 part
 *		  is present.
 *
 * @return status - Result of the initialization procedure.
 					Example: 0x0 - SPI peripheral was not initialized or
								   ADIS16227 part is not present.
 				  			 0x1 - SPI peripheral is initialized and ADIS16227
							 	   part is present.
*******************************************************************************/
unsigned char ADIS16227_Init(void)
{
    unsigned char status = 0;
    
    ADIS16227_RESET_OUT;
    ADIS16227_RESET_HIGH;
    status = SPI_Init(0, 1000000, 1, 1);
    if(ADIS16227_GetRegister16Value(ADIS16227_PROD_ID) != ADIS16227_DEFAULT_PID)
    {
        status = 0;
    }

    return(status);
}

/***************************************************************************//**
 * @brief Configures the ADIS16227_REC_CTRL register.
 *
 * @param window - Window setting.
 *                 Example: ADIS16227_RECTANGULAR - rectangular;
 *                          ADIS16227_HANNING - Hanning;
 *                          ADIS16227_FLAT_TOP - flat top.
 * @param sampleRate - Sample rate option.
 *                     Example: ADIS16227_FS - fs;
 *                              ADIS16227_FS_DIV_8 - fs  8;
 *                              ADIS16227_FS_DIV_64 - fs  64;
 *                              ADIS16227_FS_DIV_512 - fs  512.
 * @param pwrDwn - Power-down between each recording.
 *                 Example: 0 - disabled;
 *                          1 - enabled.
 * @param range - Signal range.
 *                Example: ADIS16227_RANGE_0_1G - 0 g to 1 g;
 *                         ADIS16227_RANGE_0_5G - 0 g to 5 g;
 *                         ADIS16227_RANGE_0_20G - 0 g to 20 g;
 *                         ADIS16227_RANGE_0_70G - 0 g to 70 g.
 * @param mode - The recording mode.
 *               Example: ADIS16227_MANUAL - manual;
 *                        ADIS16227_AUTOMATIC - automatic;
 *                        ADIS16227_MANUAL_TIME - manual time.
 * @return None.
*******************************************************************************/
void ADIS16227_ConfigCapture(unsigned char window,
                             unsigned char sampleRate,
                             unsigned char pwrDwn,
                             unsigned char range,
                             unsigned char mode)
{
    unsigned short recCtrl = 0;
    
    recCtrl = (ADIS16227_REC_CTRL_WINDOW(window) |
               ADIS16227_REC_CTRL_SR(sampleRate) |
               (ADIS16227_REC_CTRL_PWR_DWN_REC * pwrDwn) |
               ADIS16227_REC_CTRL_RANGE(range) |
               ADIS16227_REC_CTRL_MODE(mode));
    ADIS16227_SetRegister16Value(ADIS16227_REC_CTRL, recCtrl);
}

/***************************************************************************//**
 * @brief Reads the value of the "Data ready/busy" bit (ADIS16227_DIAG_STAT
 *        register).
 *
 * @return dataReady - Value of the "Data ready/busy" bit.
 *                     Example: 0 - Busy;
 *                              1 - Data ready.
*******************************************************************************/
unsigned char ADIS16227_DataReady(void)
{
    unsigned char dataReady = 0;
    
    dataReady = (ADIS16227_GetRegister16Value(ADIS16227_DIAG_STAT) &
                 (0x0080)) >> 7;
    
    return(dataReady);
}

/***************************************************************************//**
 * @brief Initiates a manual data capture and wait until the process is
 * finished.
 *
 * @return None.
*******************************************************************************/
void ADIS16227_Capture(void)
{
    ADIS16227_SetRegister16Value(ADIS16227_GLOB_CMD,
                                 ADIS16227_GLOB_CMD_REC_START_STOP);
    while(ADIS16227_DataReady() == 0);
}

/***************************************************************************//**
 * @brief Reads "samplesNumber" samples from each axis's buffer.
 *
 * @param x - X-axis's output data.
 * @param y - Y-axis's output data.
 * @param z - Z-axis's output data.
 * @param samplesNumber - Number of samples.
 *
 * @return None.
*******************************************************************************/
void ADIS16227_GetBuffers(unsigned short* x,
                          unsigned short* y,
                          unsigned short* z,
                          unsigned short samplesNumber)
{
    unsigned short sample = 0;
    
    ADIS16227_SetRegister16Value(ADIS16227_BUF_PNTR, 0);
    for(sample = 0; sample < samplesNumber; sample++)
    {
        x[sample] = ADIS16227_GetRegister16Value(ADIS16227_X_BUF);
    }
    ADIS16227_SetRegister16Value(ADIS16227_BUF_PNTR, 0);
    for(sample = 0; sample < samplesNumber; sample++)
    {
        y[sample] = ADIS16227_GetRegister16Value(ADIS16227_Y_BUF);
    }
    ADIS16227_SetRegister16Value(ADIS16227_BUF_PNTR, 0);
    for(sample = 0; sample < samplesNumber; sample++)
    {
        z[sample] = ADIS16227_GetRegister16Value(ADIS16227_Z_BUF);
    }
}

/***************************************************************************//**
 * @brief Reads the power supply voltage and returns the data in mV.
 *
 * @return None.
*******************************************************************************/
unsigned short ADIS16227_GetPowerSupplyInfo(void)
{
    unsigned short voltage = 0;
    
    voltage = ADIS16227_GetRegister16Value(ADIS16227_SUPPLY_OUT);
    voltage *= 1.22;
    
    return(voltage);
}

/***************************************************************************//**
 * @brief Reads the internal temperature and returns the data in degrees
 * Celsius.
 *
 * @return None.
*******************************************************************************/
char ADIS16227_GetTemperatureInfo(void)
{
    unsigned short temperature = 0;
    
    temperature = ADIS16227_GetRegister16Value(ADIS16227_TEMP_OUT);
    temperature = 0x533 - temperature;
    temperature *= 0.47;
    
    return((unsigned char)temperature);
}
